function TBar = F_qrsvn(a, b, C, nu, w)
% Integrand for computation of MVT probabilities
%
%   Given box bounds a and b (might be infinite), Cholesky factor C of the
%   correlation matrix, and degrees of freedom nu, compute the transformed MVT
%   integrand at each row of the quasi-random lattice of points w in the
%   m-dimensional unit hypercube.  If nu is Inf, vompute the transformed MVN
%   integrand with points w in the (m-1)-dimensional unit hypercube.  Return
%   the mean of the P integrand values.

N = size(w,1); % number of quasirandom points
m = length(a); % number of dimensions
if isinf(nu)
    snu = 1;
else
    snu = chiq(w(:,m),nu)./sqrt(nu);
end
d = normp(snu.*a(1));       % a is already scaled by diag(C)
emd = normp(snu.*b(1)) - d; % b is already scaled by diag(C)
T = emd;
y = zeros(N,m,class(T));
for i = 2:m
    % limit how far out in the tail y can be to a finite value to prevent
    % infinite ysum and NaN ahat=a(i)-ysum or bhat=b(i)-ysum.
    z = min(max(d + emd.*w(:,i-1), eps/2), 1-eps/2);
    y(:,i-1) = normq(z);
    ysum = y*C(:,i);
    d = normp(snu.*a(i) - ysum);       % a is already scaled by diag(C)
    emd = normp(snu.*b(i) - ysum) - d; % b is already scaled by diag(C)
    T = T .* emd;
end
TBar = sum(T,1)./length(T);
